// Функции для управления главным циклом приложения, регистрации
// колбеков для создания   приложения и его завершения.
#include <app.h>

// Функции для вывода сообщений в окно логгирования
#include <dlog.h>

// Функции для создания элементов пользовательского интерфейса
// и управления ими. Здесь находятся все виджеты, такие как: окна, кнопки и т.д.
#include <Elementary.h>

// Функции для использования колбеков аппаратной кнопки Назад и для
// использования расширенных виджетов для круглого экрана
#include <efl_extension.h>

#define PAGE_CHANGED_EVENT "active_page,changed"

typedef struct _UIData {
   Evas_Object *win;
   Evas_Object *conform;
   Evas_Object *layout;
   Evas_Object *scroller;
   Evas_Object *box;
   Evas_Object *padding_start;
   Evas_Object *padding_end;
   Evas_Object *index;
} UIData;

static const int PAGES_COUNT = 5;

// Стили для четного количества страниц
static const char *even_style[] = {
   "item/even_1",
   "item/even_2",
   "item/even_3",
   "item/even_4",
   "item/even_5",
   "item/even_6",
   "item/even_7",
   "item/even_8",
   "item/even_9",
   "item/even_10",
   "item/even_11",
   "item/even_12",
   "item/even_13",
   "item/even_14",
   "item/even_15",
   "item/even_16",
   "item/even_17",
   "item/even_18",
   "item/even_19",
   "item/even_20",
};

// Стили для нечетного количества страниц
static const char *odd_style[] = {
   "item/odd_1",
   "item/odd_2",
   "item/odd_3",
   "item/odd_4",
   "item/odd_5",
   "item/odd_6",
   "item/odd_7",
   "item/odd_8",
   "item/odd_9",
   "item/odd_10",
   "item/odd_11",
   "item/odd_12",
   "item/odd_13",
   "item/odd_14",
   "item/odd_15",
   "item/odd_16",
   "item/odd_17",
   "item/odd_18",
   "item/odd_19",
};

static void
_win_back_cb(void *data, Evas_Object *obj, void *event_info)
{
   UIData *ui = data;

   elm_win_lower(ui->win);
}

static void
_file_abs_resource_path_get(char *res_file_path, char *abs_path, int buf_size)
{
   char *res_dir_path = app_get_resource_path();

   if (res_dir_path)
     {
        snprintf(abs_path, buf_size, "%s%s", res_dir_path, res_file_path);
        free(res_dir_path);
     }
}

static void
_page_indicator_selected_set(Evas_Object *index, int page_index)
{
   Elm_Object_Item *item_to_select = elm_index_item_find(index, (void *)page_index);
   elm_index_item_selected_set(item_to_select, EINA_TRUE);
}

static Evas_Object *
_image_create(Evas_Object *parent, char *image_name)
{
   Evas_Object *image = elm_image_add(parent);

   char abs_path_to_image[PATH_MAX] = {0,};
   _file_abs_resource_path_get(image_name, abs_path_to_image, PATH_MAX);
   elm_image_file_set(image, abs_path_to_image, NULL);

   evas_object_size_hint_weight_set(image, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   evas_object_size_hint_align_set(image, EVAS_HINT_FILL, EVAS_HINT_FILL);

   evas_object_show(image);

   return image;
}

static Evas_Object *
_page_layout_create(Evas_Object *parent, int image_index)
{
   Evas_Object *page_layout = elm_layout_add(parent);
   elm_layout_theme_set(page_layout, "layout", "body_thumbnail", "default");
   evas_object_size_hint_weight_set(page_layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   evas_object_size_hint_align_set(page_layout, EVAS_HINT_FILL, EVAS_HINT_FILL);
   evas_object_show(page_layout);

   const int MAX_NAME_LENGTH = 10;
   char image_name[MAX_NAME_LENGTH] = {0,};
   snprintf(image_name, MAX_NAME_LENGTH, "%d.png", image_index);
   Evas_Object *image = _image_create(page_layout, image_name);
   elm_object_part_content_set(page_layout, "elm.icon", image);

   return page_layout;
}

static Evas_Object *
_padding_item_create(Evas_Object *parent)
{
   Evas_Object *padding = evas_object_rectangle_add(evas_object_evas_get(parent));

   evas_object_size_hint_weight_set(padding, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   evas_object_size_hint_align_set(padding, EVAS_HINT_FILL, EVAS_HINT_FILL);

   return padding;
}

static void
_layout_resize_cb(void *data , Evas *e, Evas_Object *page_layout, void *event_info)
{
   int page_width;
   int page_height;
   int container_width;
   int container_height;
   UIData *ui = data;

   evas_object_geometry_get(page_layout, NULL, NULL, &page_width, &page_height);
   evas_object_geometry_get(ui->scroller, NULL, NULL, &container_width, &container_height);

   int padding_size = (container_width - page_width) / 2;
   evas_object_size_hint_min_set(ui->padding_start, padding_size, container_height);
   evas_object_size_hint_min_set(ui->padding_end, padding_size, container_height);

   elm_scroller_page_size_set(ui->scroller, page_width, page_height);
}

static void
_box_create(UIData *ui)
{
   ui->box = elm_box_add(ui->scroller);

   const int IMAGES_COUNT = 5;
   Evas_Object *page_layout;
   for (int i = 0; i < PAGES_COUNT; ++i)
     {
        page_layout = _page_layout_create(ui->box, i % IMAGES_COUNT);
        elm_box_pack_end(ui->box, page_layout);
     }

   evas_object_event_callback_add(page_layout, EVAS_CALLBACK_RESIZE, _layout_resize_cb, ui);

   ui->padding_start = _padding_item_create(ui->box);
   elm_box_pack_start(ui->box, ui->padding_start);

   ui->padding_end = _padding_item_create(ui->box);
   elm_box_pack_end(ui->box, ui->padding_end);

   elm_box_horizontal_set(ui->box, EINA_TRUE);
   evas_object_size_hint_weight_set(ui->box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

   evas_object_show(ui->box);
}

static void
_scroll_cb(void *data, Evas_Object *scroller, void *event)
{
   static int prev_h_page = 0;
   int cur_h_page         = 0;

   elm_scroller_current_page_get(scroller, &cur_h_page, NULL);
   if (cur_h_page != prev_h_page)
     {
        UIData *ui = data;
        evas_object_smart_callback_call(ui->index, PAGE_CHANGED_EVENT, NULL);

        prev_h_page = cur_h_page;
     }
}

static void
_scroller_create(UIData *ui)
{
   ui->scroller = elm_scroller_add(ui->layout);

   elm_scroller_policy_set(ui->scroller, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_OFF);

   _box_create(ui);
   elm_object_content_set(ui->scroller, ui->box);

   evas_object_smart_callback_add(ui->scroller, "scroll", _scroll_cb, ui);

   evas_object_show(ui->scroller);
}

static void
_active_page_changed_cb(void *data, Evas_Object *index, void *event)
{
   UIData *ui     = data;
   int cur_h_page = 0;

   elm_scroller_current_page_get(ui->scroller, &cur_h_page, NULL);

   _page_indicator_selected_set(ui->index, cur_h_page);
}

static Evas_Object *
_index_create(UIData *ui)
{
   Evas_Object *index = elm_index_add(ui->layout);

   elm_object_style_set(index, "circle");
   elm_index_horizontal_set(index, EINA_TRUE);
   elm_index_autohide_disabled_set(index, EINA_TRUE);

   Eina_Bool is_even_count = PAGES_COUNT % 2 == 0;
   const int CENTER_INDEX = is_even_count ? 10 : 9;
   int start_index = CENTER_INDEX - PAGES_COUNT / 2;

   const char **style = is_even_count ? even_style : odd_style;

   for (int i = 0; i < PAGES_COUNT; ++i)
     {
        Elm_Object_Item *it = elm_index_item_append(index, NULL, NULL, (void *)i);
        elm_object_item_style_set(it, style[start_index + i]);
     }

   elm_index_level_go(index, 0);

   _page_indicator_selected_set(index, 0);

   evas_object_smart_callback_add(index, PAGE_CHANGED_EVENT, _active_page_changed_cb, ui);

   evas_object_show(index);

   return index;
}

static void
_layout_create(UIData *ui)
{
   ui->layout = elm_layout_add(ui->conform);

   elm_layout_theme_set(ui->layout, "layout", "application", "default");

   _scroller_create(ui);
   elm_object_part_content_set(ui->layout, "elm.swallow.bg", ui->scroller);

   ui->index = _index_create(ui);
   elm_object_part_content_set(ui->layout, "elm.swallow.content", ui->index);

   evas_object_show(ui->layout);
}

static void
_conformant_create(UIData *ui)
{
   ui->conform = elm_conformant_add(ui->win);

   // Говорим конформанту занимать все выделенное ему пространство
   // по горизонтали и вертикали
   evas_object_size_hint_weight_set(ui->conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

   _layout_create(ui);
   elm_object_content_set(ui->conform, ui->layout);

   evas_object_show(ui->conform);
}

static void
_window_create(UIData *ui)
{
   // Создание и настройка главного окна
   ui->win = elm_win_util_standard_add(NULL, NULL);

   // Создание конформанта – в данном случае главного контейнера в  приложении,
   // который также является интерфейсом между нашим приложением и
   // системными элементами, такими как клавиатура.
   _conformant_create(ui);

   //  Устанавливаем размер конформант-виджета такой же, как и  у окна
   elm_win_resize_object_add(ui->win, ui->conform);

   // Регистрируем функцию, которая будет вызываться
   // при нажатии аппаратной кнопки Назад
   eext_object_event_callback_add(ui->win, EEXT_CALLBACK_BACK, _win_back_cb, ui);

   // Отображаем окно всегда только после того, как отображен весь базовый
   // пользовательский интерфейс.
   evas_object_show(ui->win);
}

static bool
_app_create_cb(void *data)
{
   UIData *ui = data;

   _window_create(ui);

   return true;
}

static void
_app_terminate_cb(void *data)
{
   UIData *ui = data;

   // Перед завершением работы приложения удаляем главный виджет (окно),
   // после чего удаляются все виджеты-дети   этого окна (все виджеты нашего приложения).
   evas_object_del(ui->win);
}

int
main(int argc, char *argv[])
{
   UIData ui = {0,};
   ui_app_lifecycle_callback_s lifecycle_callbacks = {0,};

   // Регистрируем созданные нами функции, чтобы передать их в главный цикл
   // нашего приложения.
   lifecycle_callbacks.create    = _app_create_cb;
   lifecycle_callbacks.terminate = _app_terminate_cb;

   // Вызов следующей функции запускает выполнение главного цикла приложения
   return ui_app_main(argc, argv, &lifecycle_callbacks, &ui);
}
